<?php

/**
 * @file
 * Module main class.
 */

class RzModule extends RzIntegration {
  /**
   * Module system name.
   */
  protected $sModule = "rzyoutube";
  /**
   * Elements DB table.
   */
  protected $elementsDbTable;
  /**
   * Favorites DB table.
   */
  protected $favoritesDbTable;

  /**
   * Class constructor.
   */
  public function __construct($s_path, $s_url) {
    parent::__construct($s_path, $s_url);
    $this->elementsDbTable = $this->dbPrefix . $this->sModule . "_elements";
    $this->favoritesDbTable = $this->dbPrefix . $this->sModule . "_favorites";
    $this->aXmlTemplates["result"][4] = '<result value="#1#" status="#2#" param="#3#" autoplay="#4#" />';
    $this->aXmlTemplates["result"][5] = '<result value="#1#" status="#2#" page="#3#" itemsPerPage="#4#" itemsAll="#5#" />';
    $this->aXmlTemplates["error"] = array (
      2 => '<?xml version="1.0" encoding="utf-8"<ut_response status="fail"><error><code>#1#</code><description>#2#</description></error></ut_response>'
    );
    $this->aXmlTemplates["element"] = array (
      2 => '<element id="#1#"><title><![CDATA[#2#]]></title></element>',
      6 => '<element id="#1#" image="#2#" elements="#6#" param="#4#" isCategory="true"><title><![CDATA[#3#]]></title><value><![CDATA[#5#]]></value></element>',
      8 => '<element id="#1#" video="#2#" user="#3#" author="#4#" time="#5#" image="#6#" favorite="#7#"><title><![CDATA[#8#]]></title></element>',
    );
  }

  /**
   * Config generator.
   */
  public function actionConfig() {
    $s_contents = parent::actionConfig();
    $s_contents = str_replace("#joinUrl#", $this->sLoginUrl, $s_contents);
    $s_menu_location = empty($_COOKIE["RayzYoutubeMenu"]) ? "bottom" : $_COOKIE["RayzYoutubeMenu"];
    $s_contents = str_replace("#menuLocation#", $s_menu_location, $s_contents);
    $s_menu_hide = empty($_COOKIE["RayzYoutubeMenuHide"]) ? self::FALSE_VAL : $_COOKIE["RayzYoutubeMenuHide"];
    $s_contents = str_replace("#menuHide#", $s_menu_hide, $s_contents);
    $s_menu_hint = empty($_COOKIE["RayzYoutubeMenuHint"]) ? self::TRUE_VAL : $_COOKIE["RayzYoutubeMenuHint"];
    $s_contents = str_replace("#menuHint#", $s_menu_hint, $s_contents);
    return $s_contents;
  }

  /**
   * Sets config setting.
   */
  public function actionSetConfig() {
    $s_key = $this->getRequestVar("key");
    $s_value = $this->getRequestVar("value");
    if (!empty($s_key) && !empty($s_value)) {
      setcookie("RayzYoutube" . $s_key, $s_value, time() + 31536000);
    }
  }

  /**
   * Authorizes user.
   */
  public function actionUserAuthorize() {
    $s_password = $this->getRequestVar("password", "db");
    $s_value = $this->loginAdmin($this->sId, $s_password) ? self::TRUE_VAL : self::FALSE_VAL;
    $s_status = $this->loginUser($this->sId, $s_password) ? self::SUCCESS_VAL : self::FAILED_VAL;
    return $this->parseXml($this->aXmlTemplates['result'], $s_value, $s_status);
  }

  /**
   * Authorizes admin.
   */
  public function actionAdminAuthorize() {
    return $this->actionUserAuthorize();
  }

  /**
   * Creates new category.
   */
  public function actionNewCategory() {
    $s_title = $this->getRequestVar("title", "db");
    $i_category_id = $this->getRequestVar("category", "int");
    $s_id = $this->oDb->getValue("SELECT ID FROM " . $this->elementsDbTable . " WHERE Title='" . $s_title . "' AND Parent= " . $i_category_id . " LIMIT 1");
    if (empty($s_id)) {
      $s_param = $this->getRequestVar("param", "db");
      $s_value = $this->getRequestVar("value", "db");
      $this->oDb->getResult("INSERT INTO `" . $this->elementsDbTable . "`(`Title`, `Param`, `Value`, `Category`, `Parent`) VALUES('" . $s_title . "', '" . $s_param . "', '" . $s_value . "', 1, " . $i_category_id . ")");
      $i_cat_id = (int)$this->oDb->getValue("SELECT MAX(ID) FROM " . $this->elementsDbTable);
      $this->oDb->getResult("UPDATE `" . $this->elementsDbTable . "` SET `Ord`=(`Ord`+1) WHERE `Category`=1 AND `Parent`=" . $i_category_id);
      return $this->parseXml($this->aXmlTemplates["result"], $i_cat_id, self::SUCCESS_VAL);
    }
    else {
      return $this->parseXml($this->aXmlTemplates["result"], "Category with the same title already exists", self::FAILED_VAL);
    }
  }

  /**
   * Edits a category.
   */
  public function actionEditCategory() {
    $s_title = $this->getRequestVar("title", "db");
    $s_param = $this->getRequestVar("param", "db");
    $s_value = $this->getRequestVar("value", "db");
    $r_result = $this->oDb->getResult("UPDATE `" . $this->elementsDbTable . "` SET `Title`='" . $s_title . "', `Param`='" . $s_param . "', `Value`='" . $s_value . "' WHERE `ID`='" . $this->sId . "'");
    if ($r_result) {
      return $this->parseXml($this->aXmlTemplates["result"], "", self::SUCCESS_VAL);
    }
    else {
      return $this->parseXml($this->aXmlTemplates["result"], "Error changing category", self::FAILED_VAL);
    }
  }

  /**
   * Uploads category image.
   */
  public function actionUploadCategoryImage() {
    $s_extension = str_replace(".", "", $this->getRequestVar("ext"));
    $s_old_file = $this->sFilesPath . $this->getCatImage($this->sId);
    $s_file_name = $this->sFilesPath . $this->sId . "_cat." . $s_extension;
    if (is_uploaded_file($_FILES['Filedata']['tmp_name'])) {
      @unlink($s_old_file);
      move_uploaded_file($_FILES['Filedata']['tmp_name'], $s_file_name);
      @chmod($s_file_name, 0644);
    }
  }

  /**
   * Checks category image.
   */
  public function actionCheckCategoryImage() {
    $s_extension = str_replace(".", "", $this->getRequestVar("ext"));
    $s_file_name = $this->sFilesPath . $this->sId . "_cat." . $s_extension;
    if (!file_exists($s_file_name) || filesize($s_file_name) == 0) {
      return $this->parseXml($this->aXmlTemplates['result'], "Error uploading file.", self::FAILED_VAL);
    }
    else {
      return $this->parseXml($this->aXmlTemplates['result'], $this->sId . "_cat." . $s_extension, self::SUCCESS_VAL);
    }
  }

  /**
   * Removes category.
   */
  public function actionRemoveCategory() {
    $s_user = $this->getRequestVar("user", "db");
    $s_password = $this->getRequestVar("password", "db");
    if (!$this->loginAdmin($s_user, $s_password)) {
      return $this->parseXml($this->aXmlTemplates['result'], "Access error!", self::FAILED_VAL);
    }
    $i_parent_id = $this->oDb->getValue("SELECT Parent FROM " . $this->elementsDbTable . " WHERE ID='" . $this->sId . "'");
    $r_res = $this->oDb->getResult("DELETE FROM " . $this->elementsDbTable . " WHERE ID=" . $this->sId);
    @unlink($this->getCatImagePath($this->sId));
    if ($r_res) {
      $this->oDb->getResult("UPDATE " . $this->elementsDbTable . " SET Ord=(Ord-1) WHERE Category=1 AND Parent=" . $i_parent_id);
      return $this->parseXml($this->aXmlTemplates["result"], $this->sId, self::SUCCESS_VAL);
    }
    else {
      return $this->parseXml($this->aXmlTemplates["result"], "Error removing category", self::FAILED_VAL);
    }
  }

  /**
   * Changes categories order.
   */
  public function actionChangeCategoriesOrder() {
    $s_id1 = $this->getRequestVar("id1", "db");
    $r_result = $this->oDb->getResult("SELECT ID, Ord FROM " . $this->elementsDbTable . " WHERE ID='" . $this->sId . "' OR ID='" . $s_id1 . "'");
    $a_cat1 = $this->oDb->fetch($r_result);
    $a_cat2 = $this->oDb->fetch($r_result);
    $this->oDb->getResult("UPDATE " . $this->elementsDbTable . " SET Ord='" . $a_cat2['Ord'] . "' WHERE ID='" . $a_cat1['ID'] . "'");
    $this->oDb->getResult("UPDATE " . $this->elementsDbTable . " SET Ord='" . $a_cat1['Ord'] . "' WHERE ID='" . $a_cat2['ID'] . "'");
  }

  /**
   * Gets category contents.
   */
  public function actionGetCategoryContents() {
    return $this->getElements("category");
  }

  /**
   * Gets elements.
   */
  public function actionGetElements() {
    $s_mode = $this->getRequestVar("mode");
    return $this->getElements($s_mode);
  }

  /**
   * Gets elements handler.
   */
  protected function getElements($s_mode) {
    $i_page = $this->getRequestVar("page", "int");
    $i_items_per_page = $this->getRequestVar("perPage", "int");
    $i_category_id = $this->getRequestVar("category", "int");
    $s_user = $this->getRequestVar("user", "db");
    $b_paginate = FALSE;
    $s_public_factor = "WHERE Public='" . self::TRUE_VAL . "'";
    $s_count = "SELECT COUNT(ID) AS Count ";
    $s_select = "SELECT * ";
    switch ($s_mode) {
      case 'category':
        $s_sql = "";
        $s_count = "SELECT COUNT(ID) AS Count FROM " . $this->elementsDbTable . " WHERE Category=1 AND Parent=" . $i_category_id;
        $s_select = "SELECT `cats`.*, COUNT(`els`.`ID`) AS `Count` FROM `" . $this->elementsDbTable . "` AS `cats` LEFT JOIN `" . $this->elementsDbTable . "` AS `els` ON `cats`.`ID`=`els`.`Parent` WHERE `cats`.`Category`=1 AND `cats`.`Parent`=" . $i_category_id . " GROUP BY `cats`.`ID` ORDER BY `cats`.`Category` DESC, `cats`.`Ord` ASC, `cats`.`ID` DESC";
        break;
      case 'favorites':
        $s_count = "SELECT COUNT(El.ID) AS Count ";
        $s_select = "SELECT El.* ";
        $s_sql = "FROM " . $this->elementsDbTable . " AS El INNER JOIN " . $this->favoritesDbTable . " AS Fv WHERE El.Category=0 AND El.ID=Fv.Element AND Fv.User='" . $s_user ."' ORDER BY Fv.ID DESC";
        $s_user = "";
        break;
      case 'featured':
      default:
        $b_paginate = TRUE;
        $s_sql = "FROM `" . $this->elementsDbTable . "` " . $s_public_factor . " AND `Category`=0 ORDER BY `ID` DESC";
        break;
    }
    $i_count = (int)@$this->oDb->getValue($s_count . $s_sql);
    if (!$b_paginate) {
      $i_items_per_page = $i_count;
    }
    $r_result = @$this->oDb->getResult($s_select . $s_sql . ($b_paginate ? $this->getPagination($i_page, $i_items_per_page) : ""));
    if (!$r_result) {
      return $this->parseXml($this->aXmlTemplates["result"], "msgErrorGetVideos", self::FAILED_VAL);
    }
    $s_contents = $this->parseXml($this->aXmlTemplates["result"], "", self::SUCCESS_VAL, $i_page, $i_items_per_page, $i_count);
    $s_elements = "";
    $i_current_time = time();
    while (($a_element = $this->oDb->fetch($r_result)) !== NULL) {
      if ($a_element['Category']) {
        $s_elements .= $this->parseXml($this->aXmlTemplates['element'], $a_element['ID'], $this->getCatImage($a_element['ID']), stripslashes($a_element['Title']), $a_element['Param'], $a_element['Value'], $a_element['Count']);
      }
      else {
        $s_favorite = (!empty($s_user) && $this->isFavorite($s_user, $a_element['ID'])) ? self::TRUE_VAL : self::FALSE_VAL;
        $s_featured = $a_element['Featured'] > 0 ? self::TRUE_VAL : self::FALSE_VAL;
		$a_user = $this->getUserInfo($a_element['User']);
        $s_elements .= $this->parseXml($this->aXmlTemplates["element"], $a_element['ID'], $a_element['Video'], $a_user['nick'], $a_element['Author'], $a_element['Length'], $a_element['Image'], $s_favorite, stripslashes($a_element['Title']));
      }
    }
    $s_contents .= $this->makeGroup($s_elements, "elements");
    return $s_contents;
  }

  /**
   * Gets elements handler.
   */
  protected function actionAddVideo() {
    $s_public = $this->getRequestVar("public", "strbool");
    $i_length = $this->getRequestVar("length", "int");
    $s_user = $this->getRequestVar("user", "db");
    $s_author = $this->getRequestVar("author", "db");
    $s_video = $this->getRequestVar("video", "db");
    $s_title = $this->getRequestVar("title", "db");
    $s_image = $this->getRequestVar("image", "db");
    $a_video = $this->oDb->getArray("SELECT `ID`, `Public` FROM `" . $this->elementsDbTable . "` WHERE `Video`='" . $s_video . "' LIMIT 1");
    if (!empty($a_video['ID'])) {
      $this->sId = $a_video['ID'];
    }
    if (empty($this->sId)) {
      $this->oDb->getResult("INSERT INTO `" . $this->elementsDbTable . "`(`Video`, `User`, `Author`, `Title`, `Length`, `Image`, `Public`) VALUES('" . $s_video . "', '" . $s_user . "', '" . $s_author . "', '" . $s_title . "', '" . $i_length . "', '" . $s_image . "', '" . $s_public . "')");
      $this->sId = (int)$this->oDb->getValue("SELECT MAX(ID) FROM " . $this->elementsDbTable);
    }
    elseif ($s_public == self::TRUE_VAL && $a_video['Public'] == self::FALSE_VAL) {
      $this->oDb->getResult("UPDATE `" . $this->elementsDbTable . "` SET `Public`='" . $s_public . "', `User`='" . $s_user . "' WHERE `Video`='" . $s_video . "' AND `Public`='" . self::FALSE_VAL . "' LIMIT 1");
    }
    if ($s_public == self::TRUE_VAL) {
      return $this->parseXml($this->aXmlTemplates['result'], $this->sId, self::SUCCESS_VAL);
    }
    return $this->actionAddFavorite();
  }

  /**
   * Adds favorite.
   */
  public function actionAddFavorite() {
    $s_user = $this->getRequestVar("user", "db");
    if (!$this->isFavorite($s_user, $this->sId)) {
      $this->oDb->getResult("INSERT INTO `" . $this->favoritesDbTable . "`(`User`, `Element`) VALUES('" . $s_user . "', '" . $this->sId . "')");
    }
    return $this->parseXml($this->aXmlTemplates['result'], $this->sId, self::SUCCESS_VAL);
  }

  /**
   * Adds favorite.
   */
  public function actionRemoveFavorite() {
    $s_user = $this->getRequestVar("user", "db");
    $this->oDb->getResult("DELETE FROM `" . $this->elementsDbTable . "` WHERE `User`='" . $s_user . "' AND `ID`='" . $this->sId . "' AND `Public`='" . self::FALSE_VAL . "' LIMIT 1");
    $r_res = $this->oDb->getResult("DELETE FROM `" . $this->favoritesDbTable . "` WHERE `User`='" . $s_user . "' AND `Element`='" . $this->sId . "'");
    if ($r_res) {
      return $this->parseXml($this->aXmlTemplates["result"], "", self::SUCCESS_VAL);
    }
    else {
      return $this->parseXml($this->aXmlTemplates["result"], "Error removing favorite", self::FAILED_VAL);
    }
  }

  /**
   * Removes station.
   */
  public function actionRemove() {
    $i_start = $this->getRequestVar("start", "int");
    $s_user = $this->getRequestVar("user", "db");
    $s_password = $this->getRequestVar("password");
    if (!$this->loginAdmin($s_user, $s_password)) {
      return $this->parseXml($this->aXmlTemplates['result'], "Access error!", self::FAILED_VAL);
    }
    $this->oDb->getResult("DELETE FROM `" . $this->favoritesDbTable . "` WHERE `Element`='" . $this->sId . "'");
    $r_res = $this->oDb->getResult("DELETE FROM `" . $this->elementsDbTable . "` WHERE `ID`='" . $this->sId . "' LIMIT 1");
    if ($r_res) {
      $s_contents = $this->parseXml($this->aXmlTemplates["result"], "", self::SUCCESS_VAL);
      if ($i_start > 0) {
        $a_user = $this->getUserInfo($s_user);
        $a_element = $this->oDb->getArray("SELECT * FROM `" . $this->elementsDbTable . "` WHERE `Public`='" . self::TRUE_VAL . "' AND `Category`=0 ORDER BY `ID` DESC LIMIT " . ($i_start-1) . ",1");
        $s_favorite = (!empty($s_user) && $this->_isFavorite($s_user, $a_element['ID'])) ? self::TRUE_VAL : self::FALSE_VAL;
        $s_contents .= $this->parseXml($this->aXmlTemplates["element"], $a_element['ID'], $a_element['Video'], $a_user['nick'], $a_element['Author'], $a_element['Length'], $a_element['Image'], $s_favorite, stripslashes($a_element['Title']));
      }
    }
    else {
      $s_contents = $this->parseXml($this->aXmlTemplates["result"], "Error removing video", self::FAILED_VAL);
    }
    return $s_contents;
  }

  /**
   * Gets category image.
   */
  protected function getCatImage($s_category_id) {
    if (file_exists($this->sFilesPath . $s_category_id . "_cat.jpg")) {
      return $s_category_id . "_cat.jpg";
    }
    if (file_exists($this->sFilesPath . $s_category_id . "_cat.png")) {
      return $s_category_id . "_cat.png";
    }
    if (file_exists($this->sFilesPath . $s_category_id . "_cat.gif")) {
      return $s_category_id . "_cat.gif";
    }
    return "cat.png";
  }

  /**
   * Gets category image path.
   */
  protected function getCatImagePath($s_category_id) {
    $s_cat = $this->getCatImage($s_category_id);
    if ($s_cat == "cat.png") {
      return "";
    }
    else {
      return $this->sFilesPath . $s_cat;
    }
  }

  /**
   * Gets pagination.
   */
  protected function getPagination($i_page = 0, $i_items_per_page = 25)	{
    return " LIMIT " . $i_page * $i_items_per_page . ", " . $i_items_per_page;
  }

  /**
   * Checks if the given element id is favorited by given user.
   */
  protected function isFavorite($s_user, $s_element_id) {
    $s_favoriteId = $this->oDb->getValue("SELECT ID FROM " . $this->favoritesDbTable . " WHERE User='" . $s_user . "' AND Element='" . $s_element_id . "' LIMIT 1");
    return !empty($s_favoriteId);
  }
}
